var utils = require("../utils");

/**
 * Contains the mapping between partial file path to git path.
 */
function FilesContainer(logger) {
    this.logger = logger;
    this.partialToGitMap = {};
}


/**
 * Add fille to filess map, each files has keys per each optional sub path.
 * For example: given file 'foo/src/index.js', it gets the following keys:
 * 1. 'foo/src/index.js'
 * 2. 'src/index.js'
 * 3. 'index.js'
 * @param scmFile
 */
FilesContainer.prototype.addFile = function(scmFile) {
    scmFile = utils.adjustPathSlashes(scmFile);
    var parts = scmFile.split("/");
    for (var i = parts.length; i > 0; i--) {
        var partsToSlice = parts.length - i;
        var key = parts.slice(partsToSlice).join("/");
        this.partialToGitMap[key] = this.partialToGitMap[key] || [];
        this.partialToGitMap[key].push(scmFile);
    }
};


/**
 *
 * @param partialPath
 * @returns {[]}
 */
FilesContainer.prototype.getScmPath = function(partialPath){
    partialPath = utils.adjustPathSlashes(partialPath);
    var parts = partialPath.split("/");
    for (var i = parts.length; i > 0; i--) {
        var partsToSice = parts.length - i;
        var key = parts.slice(partsToSice).join("/");
        var gitPathArr = this.partialToGitMap[key];
        if(gitPathArr && gitPathArr.length === 1){
            break;
        }
    }
    if(gitPathArr == null){
        this.logger.warn(partialPath + " has no match to any git file, consider using SL_sourceRoot");
        return null;
    }
    return gitPathArr;
}
module.exports = FilesContainer;