var jwtDecode = require('jwt-decode');
var globalErrorHandler = require('./global-error-handler');
var JWTPrefix = "eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.";

module.exports = {
    parseToken: function parseToken(parsedToken, parsedArgs) {
        try {
            if (parsedArgs.customerid && parsedArgs.customerid.indexOf(JWTPrefix) == 0) {
                parsedToken = parsedArgs.customerid;
                parsedArgs.token = parsedToken;
            }
            if (!parsedToken) {
                throw new Error("Token Must Be Given");
            }
            var tokenData = jwtDecode(parsedToken);
            if (!tokenData["x-sl-server"]) {
                throw new Error("Doesn't Contain Server");
            }
            if (!tokenData["subject"]) {
                throw new Error("Doesn't Contain Subject");
            }
            var customerId = tokenData["subject"];
            var subjectParts = tokenData["subject"].split("@");
            if (subjectParts.length >= 1) {
                customerId = subjectParts[0];
            }
            parsedArgs.server = tokenData["x-sl-server"];
            parsedArgs.customerid = customerId;
            parsedArgs.isValidToken = true;
        } catch (err) {
            globalErrorHandler.setLastError(err, "Token Is Invalid. ");
            console.error("Token Is Invalid. " + err.stack);
            parsedArgs.isValidToken = false;
        }
    },

    adjustPathSlashes: function adjustPathSlashes(path) {
        path = (path || "").replace(/\\/g, '/');
        return path;
    },

    expandCommaSeparatedValues: function expandCommaSeparatedValues(arr, delim) {
        delim = delim || ',';
        arr = arr || [];
        var ret = [];
        arr.forEach(function (item) {
            var parts = item.split(delim);
            ret = ret.concat(parts);
        })
        return ret;
    },

    getMissingProperties: function getMissingFields(fieldsMap){
        var missingFields = [];
        Object.keys(fieldsMap).forEach(function (key){
            if (fieldsMap[key]) {
                missingFields.push(key);
            }
        });
        return missingFields.toString();
    },
    isMarkupFile: function (extension) {
        return this.isArrayContainsMarkupFile([extension]);
    },
    isArrayContainsMarkupFile: function (extensionsArr) {
        var MARKUP_FILES_EXTENSIONS_MAP = {'.ftl': true, '.html': true};
        if(!Array.isArray(extensionsArr)){return false}
        for(var i = 0; i < extensionsArr.length; i++) {
            var ext = extensionsArr[i].toLowerCase();
            if(MARKUP_FILES_EXTENSIONS_MAP[ext] === true){
                return true;
            }
        }
        return false
    },
    distinctArray: function (value, index, array) {
        return array.indexOf(value) === index;
    },
    cloneObject: function (source, logger) {
        try {
            return JSON.parse(JSON.stringify(source));
        }catch (e){
            logger.error("Failed to clone, returning shallow ");
            return null;
        }
    }
};